@@ -24,8 +24,8 @@ class NSGA3:
     duplicate solutions, duplicates are removed at each generation and only unique
     solutions are retained for the next generation.
     
-    A few metrics are calculated by this algorithm, including mutual domination
-    rate (MDR), non-dominated hypervolume (HV), and the % of filled reference
+    A couple metrics are calculated by this algorithm, including mutual domination
+    rate (MDR) and the % of filled reference
     directions. These can be used to assess the success of the optimization.
     However, no early stopping for convergence is implemented.
     
@@ -66,6 +66,8 @@ class NSGA3:
     record_gens : int, optional
         How often to record statistics and calculate convergence metrics. The default is
         10 generations.
+    verbose : bool
+        Whether to print progress to screen. The default is True.
     seed : int, optional
         Random seed. The default is None.
 
@@ -74,7 +76,7 @@ class NSGA3:
     :class:`gatf.population.Population`
         Optimization results as dict-like key/value pairs including 'X', 'fitness',
         and 'statistics'. The statistics key contains a :class:`population.Statistics`
-        object with the keys 'gen', 'mdr', 'hv', and 'refs'.
+        object with the keys 'gen', 'mdr', and 'refs'.
     
     Notes
     -----
@@ -150,11 +152,14 @@ class NSGA3:
                  mutation_eta=1.0,
                  crossover_eta=1.0,
                  record_gens=10,
+                 verbose=True,
                  seed=None):
         
         if seed is not None:
             tf.random.set_seed(seed)
         
+        self.verbose = verbose
+        
         assert n_objectives > 1
         assert tournament_size < population_size
 
@@ -194,7 +199,7 @@ class NSGA3:
         # Generate reference directions
         ref_dirs = population.uniform_reference_points(n_objectives, partitions)
         
-        print('Population size: {}, reference directions: {}, partitions: {}'.format(population_size,int(n_references),partitions))
+        if self.verbose: print('Population size: {}, reference directions: {}, partitions: {}'.format(population_size,int(n_references),partitions))
         
         # Normalize reference directions so that each is a unit vector
         self.ref_dirs = ref_dirs / tf.norm(ref_dirs, axis=1, keepdims=True)
@@ -249,31 +254,6 @@ class NSGA3:
         
         return M1 - M2
     
-    @tf.function(input_signature=(tf.TensorSpec(shape=[None,None], dtype=tf.float64),)) 
-    def _calculate_hypervolume(self,f):
-        '''
-        Calculate the non-dominated hypervolume
-        '''
-        # Find indices of non-dominated front
-        nd_idx,_ = self._find_nondominated(f)
-        # Actually get non-dominated front for each
-        f_nd = tf.gather(f,nd_idx,axis=0)
-        # Normalize f_last
-        f_nd = self._adaptive_normalization(f_nd)
-        # Sort first column ascending
-        sort_idx = tf.argsort(f_nd[:,0], axis=0)
-        f_nd = tf.gather(f_nd, sort_idx, axis=0)
-        # Separate first column from remaining and calcuate distance to reference point
-        D1 = tf.reshape(self.hv_ref[:,0] - f_nd[:,0],(-1,1))
-        # Concat reference point then select columns 1:
-        D2 = tf.concat([self.hv_ref,f_nd], axis=0)[:,1:]
-        # Calculate difference per adjacent row for each column
-        #D2 = tf.abs(D2[:-1,:]-D2[1:,:])
-        D2 = D2[:-1,:]-D2[1:,:]
-        # Concat columns
-        D = tf.concat([D1,D2], axis=-1)
-        return tf.reduce_sum(tf.math.reduce_prod(D,axis=1))
-    
     @tf.function(input_signature=(tf.TensorSpec(shape=[None,None], dtype=tf.float64),)) 
     def _calculate_filled_references(self,f):
         '''
@@ -476,16 +456,14 @@ class NSGA3:
         # Calculate statistics before starting optimization
         parents = self.population['X']
         f_new = self.obj_func(parents)
-        hv = self._calculate_hypervolume(f_new)
         mdr = self._calculate_mdr(f_new,f_new)
         refs = self._calculate_filled_references(f_new)
 
-        print('Generation 0 – HV {} – MDR {} – Ref dirs {}%     '.format(np.round(hv.numpy(),2),np.round(mdr.numpy(),2),np.round(100*refs.numpy())), end = '\r')
+        if self.verbose: print('Generation 0 – MDR {} – Ref dirs {}%     '.format(np.round(mdr.numpy(),2),np.round(100*refs.numpy())), end = '\r')
 
         # Add statistics
         self.population.statistics['gen'] = 0
         self.population.statistics['mdr'] = mdr
-        self.population.statistics['hv'] = hv
         self.population.statistics['refs'] = refs
         
         for gen in range(1, self.generations+1):
@@ -513,16 +491,14 @@ class NSGA3:
             if gen % self.record_gens == 0: 
                 
                 # For statistics
-                hv = self._calculate_hypervolume(f_new)
                 mdr = self._calculate_mdr(f_prev,f_new)
                 refs = self._calculate_filled_references(f_new)
 
-                print('Generation {} – HV {} – MDR {} – Ref dirs {}%     '.format(gen,np.round(hv.numpy(),2),np.round(mdr.numpy(),2),np.round(100*refs.numpy())), end = '\r')
+                if self.verbose: print('Generation {} – MDR {} – Ref dirs {}%     '.format(gen,np.round(mdr.numpy(),2),np.round(100*refs.numpy())), end = '\r')
 
                 # Add statistics
                 self.population.statistics['gen'] = gen
                 self.population.statistics['mdr'] = mdr
-                self.population.statistics['hv'] = hv
                 self.population.statistics['refs'] = refs
                 
             
@@ -531,7 +507,7 @@ class NSGA3:
         self.population['X'] = tf.gather(parents, nd_idx, axis=0)
         self.population['fitness'] = tf.gather(f_new, nd_idx, axis=0)
         
-        print('\nComplete – non-dominated front size: {}'.format(nd_idx.shape[0]))
+        if self.verbose: print('\nComplete – non-dominated front size: {}'.format(nd_idx.shape[0]))
         return self.population
 
     
@@ -591,6 +567,8 @@ class muPlusLambda:
     record_gens : int, optional
         How often to record statistics and check for convergence. The default is
         10 generations.
+    verbose : bool
+        Whether to print progress to screen. The default is True.
     seed : int, optional
         Random seed. The default is None.
 
@@ -670,11 +648,14 @@ class muPlusLambda:
                  func_tolerance=0.0,
                  position_tolerance=1e-8,
                  record_gens=10,
+                 verbose=True,
                  seed=None):
     
         if seed is not None:
             tf.random.set_seed(seed)
-            
+           
+        self.verbose = verbose
+        
         assert tournament_size < population_size
         
         # Make 1D tensors of bounds
@@ -807,7 +788,7 @@ class muPlusLambda:
         self.population.statistics['fitness_mean'] = f_mean
         self.population.statistics['fitness_best'] = f_best
         
-        print('Generation 0 – min {} - mean {} – max {} – best {}   '.format(np.round(f_min.numpy(),2),np.round(f_mean.numpy(),2),np.round(f_max.numpy(),2),np.round(tf.squeeze(f_best).numpy(),2)), end = '\r')
+        if self.verbose: print('Generation 0 – min {} - mean {} – max {} – best {}   '.format(np.round(f_min.numpy(),2),np.round(f_mean.numpy(),2),np.round(f_max.numpy(),2),np.round(tf.squeeze(f_best).numpy(),2)), end = '\r')
         
         for gen in range(1, self.generations+1):
             # Apply tournament selection
@@ -840,14 +821,14 @@ class muPlusLambda:
                 self.population.statistics['fitness_mean'] = f_mean
                 self.population.statistics['fitness_best'] = f_best
                 
-                print('Generation {} – min {} - mean {} – max {} – best {}   '.format(gen,np.round(f_min.numpy(),2),np.round(f_mean.numpy(),2),np.round(f_max.numpy(),2),np.round(tf.squeeze(f_best).numpy(),2)), end = '\r')
+                if self.verbose: print('Generation {} – min {} - mean {} – max {} – best {}   '.format(gen,np.round(f_min.numpy(),2),np.round(f_mean.numpy(),2),np.round(f_max.numpy(),2),np.round(tf.squeeze(f_best).numpy(),2)), end = '\r')
                 
                 # Check convergence and break if converged
                 if f_max-f_min < self.func_tolerance:
-                    print('\nFitness converged at gen {}'.format(gen), end = '\r')
+                    if self.verbose: print('\nFitness converged at gen {}'.format(gen), end = '\r')
                     break
                 elif tf.reduce_all(X_max-X_min < self.position_tolerance):
-                    print('\nPositions converged at gen {}'.format(gen), end = '\r')
+                    if self.verbose: print('\nPositions converged at gen {}'.format(gen), end = '\r')
                     break
         
         # Update population object
@@ -856,7 +837,7 @@ class muPlusLambda:
         self.population['X_best'] = X_best
         self.population['fitness_best'] = f_best
         
-        print('\nComplete')
+        if self.verbose: print('\nComplete')
         return self.population
         
         
@@ -907,6 +888,8 @@ class PSO:
     record_gens : int
         How often to record statistics and check for convergence. The default is
         10 generations.
+    verbose : bool
+        Whether to print progress to screen. The default is True.
     seed : int, None
         Random seed. The default is None.
 
@@ -984,11 +967,14 @@ class PSO:
                  func_tolerance=0.0,
                  position_tolerance=1e-8,
                  record_gens=10,
+                 verbose=True,
                  seed=None):
     
         if seed is not None:
             tf.random.set_seed(seed)
         
+        self.verbose = verbose
+        
         # Make 1D tensors of bounds
         if isinstance(upperbound,Number):
             upperbound = tf.constant(upperbound, shape=n_variables, dtype=tf.float64)
@@ -1122,7 +1108,7 @@ class PSO:
         self.particles.statistics['fitness_mean'] = f_mean
         self.particles.statistics['fitness_best'] = tf.squeeze(gbest_f)
         
-        print('Generation 0 – min {} - mean {} – max {} – best {}   '.format(np.round(f_min.numpy(),2),np.round(f_mean.numpy(),2),np.round(f_max.numpy(),2),np.round(tf.squeeze(gbest_f).numpy(),2)), end = '\r')
+        if self.verbose: print('Generation 0 – min {} - mean {} – max {} – best {}   '.format(np.round(f_min.numpy(),2),np.round(f_mean.numpy(),2),np.round(f_max.numpy(),2),np.round(tf.squeeze(gbest_f).numpy(),2)), end = '\r')
         
         for gen in range(1, self.generations+1):
             
@@ -1149,14 +1135,14 @@ class PSO:
                 self.particles.statistics['fitness_mean'] = f_mean
                 self.particles.statistics['fitness_best'] = tf.squeeze(gbest_f)
                 
-                print('Generation {} – min {} - mean {} – max {} – best {}   '.format(gen,np.round(f_min.numpy(),2),np.round(f_mean.numpy(),2),np.round(f_max.numpy(),2),np.round(tf.squeeze(gbest_f).numpy(),2)), end = '\r')
+                if self.verbose: print('Generation {} – min {} - mean {} – max {} – best {}   '.format(gen,np.round(f_min.numpy(),2),np.round(f_mean.numpy(),2),np.round(f_max.numpy(),2),np.round(tf.squeeze(gbest_f).numpy(),2)), end = '\r')
                 
                 # Check convergence and break if converged
                 if f_max-f_min < self.func_tolerance:
-                    print('\nFitness converged at gen {}'.format(gen), end = '\r')
+                    if self.verbose: print('\nFitness converged at gen {}'.format(gen), end = '\r')
                     break
                 elif tf.reduce_all(X_max-X_min < self.position_tolerance):
-                    print('\nPositions converged at gen {}'.format(gen), end = '\r')
+                    if self.verbose: print('\nPositions converged at gen {}'.format(gen), end = '\r')
                     break
         
         # Update population object
@@ -1165,7 +1151,7 @@ class PSO:
         self.particles['X_best'] = gbest_X
         self.particles['fitness_best'] = gbest_f
         
-        print('\nComplete')
+        if self.verbose: print('\nComplete')
         return self.particles
         
         
